#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   flowprofile.py
@Time    :   2019/03/19
@Author  :   Yaron Huang 
@Version :   1.0
@Contact :   yaronhuang@qq.com
@Desc    :   流量配置：增加端口、删除端口、修改端口流量限制     
             文件示例："8080":{"limit":1024,"used":500}
'''

import os
import json

class FlowProfile(object):
    def __init__(self, fileName = None):
        self.profile = {}
        try:
            if fileName != None:
                str = None
                with open(fileName, 'r+') as fd:
                    str = fd.read(str)
                self.profile = json.loads(str)
        except:
            pass

    def addPort(self, port, limit):
        port  = str(port)
        limit = int(limit)
        used  = 0
        if self.profile.get(port) != None:
            self.profile[port]['limit'] = limit
            self.profile[port]['changeLimit'] = 1
        else:
            self.profile[port] = {'limit':limit, 'used':0}

    def delPort(self, port):
        port = str(port)
        if self.profile.get(port) != None:
            self.profile.pop(port)

    def resetPortUsed(self, port):
        port = str(port)
        if self.profile.get(port) != None:
            self.profile[port]['used'] = 0
            self.profile[port]['reset'] = 1

    def save(self, fileName):
        try:
            str   = json.dumps(self.profile)
            index = fileName.rfind('/')
            path  = fileName[0:index+1]
            if not os.path.exists(path):
                os.makedirs(path)
            with open(fileName, 'w+') as fd:
                fd.write(str)
        except:
            return False
        return True


    def __isPortReset(self, profile, port):
        if profile[port].get('reset') != None:
            if profile[port]['reset'] == 1:
                return True
        return False

    def __isPortChangeLimit(self, profile, port):
        if profile[port].get('changeLimit') != None:
            if profile[port]['changeLimit'] == 1:
                return True
        return False

    def reload(self, fileName):
        """
        #Func    :   重载主要是读取开关,包括:增删端口、重置流量、修改上限       
        """
        if fileName == None:
            return
        try:
            str = None
            with open(fileName, 'r+') as fd:
                str = fd.read(str)
            value = json.loads(str)
            for port in value:
                if self.profile.get(port) != None:
                    self.addPort(port, value[port]['limit'])
                elif self.__isPortReset(value, port):
                    self.profile[port]['used'] = 0
                    self.profile[port]['reset'] = 0
                elif self.__isPortChangeLimit(value, port):
                    self.profile[port]['limit'] = value[port]['limit']
                    self.profile[port]['changeLimit'] = 0
            for port in self.profile:
                if value.get(port) != None:
                    self.delPort(port)
        except:
            pass
    






import os
import sys
import time
import subprocess

from aigpy import cmdHelper
from aigpy import fileHelper
from aigpy import convertHelper
from aigpy import systemHelper

from ss_py.ssprofile   import SSProfile
from ss_py.flowprofile import FlowProfile
from ss_py.iptable     import IPTable

PATH_BASE          = '/etc/ss_py/'
FILE_PROFILE       = PATH_BASE + 'profile.json'
FILE_SSERVER_PID   = PATH_BASE + 'ssserverpid.txt'
FILE_COUNT_PID     = PATH_BASE + 'countpid.txt'
FILE_FLOW          = PATH_BASE + 'flow.json'
FLAG_COUNT_PROCESS = 'FLAG_COUNT'

class SSTool(object):
    def __init__(self):
        self.profile = SSProfile(FILE_PROFILE)
        self.flow    = FlowProfile(FILE_FLOW)
   

    def _checkPortRanges(self, port):
        port = int(port)
        if port > 0 and port <= 65536:
            return True
        return False

    def __getPidByFile(self, isCountFile = False):
        if isCountFile:
            pid = fileHelper.getFileContent(FILE_COUNT_PID)
        else:
            pid = fileHelper.getFileContent(FILE_SSERVER_PID)
        if pid == "":
            return -1
        pid.strip()
        pid.strip('\n')
        pid = int(pid)
        return pid

    def __startCount(self):
        os.system('ss-py ' + FLAG_COUNT_PROCESS + ' & echo $! > '+FILE_COUNT_PID)
    
    def __stopCount(self):
        pid = self.__getPidByFile(True)
        if pid == -1:
            return False
        array = systemHelper.getProcessID('ss-py')
        if pid in array:
            systemHelper.killProcess(pid)
            os.remove(FILE_COUNT_PID)
            return True
        return False

    def startSS(self):
        if self.isSSOpen():
            return True
        cmd = 'ssserver -qq -c ' + FILE_PROFILE + ' 2>/dev/null >/dev/null & echo $! > ' + FILE_SSERVER_PID
        res = subprocess.call(cmd, shell=True)
        # 等待一下,要给上面的进程一段时间判断启动是否正常
        time.sleep(0.5)
        if self.isSSOpen() == False:
            return False
        self.__startCount()
        return True

    def stopSS(self):
        pid = self.__getPidByFile()
        if pid == -1:
            return False
        array = systemHelper.getProcessID('`basename ssserver`')
        if pid in array:
            systemHelper.killProcess(pid)
            os.remove(FILE_SSERVER_PID)
            self.__stopCount()
            return True
        return False
    
    def isSSOpen(self):
        pid = self.__getPidByFile()
        if pid == -1:
            return False
        array = systemHelper.getProcessID('`basename ssserver`')
        if pid in array:
            return True
        return False

    def getAnotherSSPID(self):
        array = systemHelper.getProcessID('`basename ssserver`')
        if len(array) == 0:
            return []
        pid = self.__getPidByFile()
        if pid == -1:
            return array
        if str(pid) in array:
            array.remove(str(pid))
        return array
    
    def killAnotherSSPID(self):
        array = self.getAnotherSSPID()
        for item in array:
            systemHelper.killProcess(item)

    def addDelPort(self, isAdd, port, pwd, limit):
        if self._checkPortRanges(port) == False:
            return False
        if isAdd:
            self.profile.addPort(port, pwd)
            self.flow.addPort(port, limit)
        else:
            self.profile.delPort(port)
            self.flow.delPort(port)

        self.profile.save(FILE_PROFILE)
        self.flow.save(FILE_FLOW)
        if self.isSSOpen():
            self.stopSS()
            self.startSS()
    
    def printPorts(self):
        if len(self.profile.ports) <= 0:
            cmdHelper.myprint('[错误] ',cmdHelper.TextColor.Red)
            print('未设置端口')
            return

        self.flow = FlowProfile(FILE_FLOW)
        cols =['Port','Password','Limit','Remaining']
        rows = []
        for port, pwd in self.profile.ports.items():
            limit = self.flow.profile[port]['limit']
            used  = self.flow.profile[port]['used']
            rema  = limit - used
            if rema < 0:
                rema = 0
            limit = convertHelper.convertStorageUnitToString(limit, 'byte')
            rema  = convertHelper.convertStorageUnitToString(rema, 'byte')
            rows.append([port, pwd, limit, rema])
        cmdHelper.showTable(cols, rows, cmdHelper.TextColor.Red, cmdHelper.TextColor.Green)
           
    def printStatus(self):
        print('[状态] ', end='')
        if self.isSSOpen():
            cmdHelper.myprint('已启动\n',cmdHelper.TextColor.Green)
        else:
            cmdHelper.myprint('停止\n',cmdHelper.TextColor.Red)

    def isCountProcess(self):
        return cmdHelper.findInArgv(FLAG_COUNT_PROCESS) != None
    


class CountTool(object):
    def __init__(self):
        self.flow    = FlowProfile(FILE_FLOW)
        self.iptable = IPTable()
        self.iptable.initChain()
        for port in self.flow.profile:
            self.iptable.addPortRule(int(port))
        
    def start(self):
        tmpNum = 0
        timeOfCount = 0.5
        while True:
            # 计算增加的流量,更新增量到self.flow.profile中
            increase = self.iptable.getPortsFlowIncrease()
            for port in increase:
                if port in self.flow.profile:
                    self.flow.profile[port]['used'] = increase[port]
            # 休眠
            time.sleep(timeOfCount)
            # 重载流量文件,判断SSTool有没有修改流量文件
            # 检查流量,查看是否封闭端口或启动端口
            tmpNum = tmpNum + 1
            if tmpNum*timeOfCount >= 10:
                tmpNum = 0
                self.flow.reload(FILE_FLOW)
                self.flow.save(FILE_FLOW)
                for port in self.flow.profile:
                    rema = self.flow.profile[port]['limit'] - self.flow.profile[port]['used']
                    if rema > 0:
                        self.iptable.addPortRule(port)
                        self.iptable.deletePortRule(port, True)
                    else:
                        self.iptable.addPortRule(port,True)
                        self.iptable.deletePortRule(port, False)